==> switch_theme-1.0.0.tar.gz: switch_theme/src/Theme/ThemeNegotiator.php <==
<?php
/**
 * @author racol <contact@rcowebdev.com>
 * 
 * Contains \Drupal\switch_theme\Theme\ThemeNegotiator
 */
namespace Drupal\switch_theme\Theme;

use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Theme\ThemeNegotiatorInterface;
use Drupal\switch_theme\SwitchthemeManager;

class ThemeNegotiator implements ThemeNegotiatorInterface {
    /**
     * Theme to negociate
     * 
     * @var string
     */
    protected $theme = "";

    /**
     * Role code of administrator
     * 
     * @todo Administrator role code can be changed, must get it with a query.
     * 
     * @var string
     */
    protected $administratorRoleCode = 'administrator';

    /**
     * The switch theme manager.
     *
     * @var SwitchthemeManager
     */
    protected $switchemeManager;

    /**
     * Construct
     *
     * @param SwitchthemeManager $switchemeManager The switch theme manager
     * 
     * @return void
     */
    public function __construct(SwitchthemeManager $switchemeManager) {
        $this->switchemeManager = $switchemeManager;
    }

    /**
     * Applies theme conditions, "return X" is not a mistake, we want to leave the method as soon as possible.
     *
     * @param RouteMatchInterface $route_match
     * 
     * @return boolean
     */
    public function applies(RouteMatchInterface $route_match) {

        $current_user = \Drupal::currentUser();
        $userRoles = $current_user->getRoles();

        /**
         * The current user is an administrator, the theme will be the default admin theme choosed
         */
        if (in_array($this->administratorRoleCode, $userRoles)) {
            $this->theme = \Drupal::config('system.theme')->get('admin');
            return true;
        }

        $switchthemes = $this->switchemeManager->getSwitchThemes();
        
        /**
         * Get query parameters to be sure ThemeNegociator will apply or not the change of the theme
         * 
         * @see https://www.php.net/manual/fr/function.strtok.php
         */
        $requestUri = strtok(\Drupal::request()->getRequestUri(), '?');

        foreach ($switchthemes as $theme) {
            /**
             * @see https://www.php.net/manual/fr/function.array-intersect.php
             */
            $intersect = array_intersect($userRoles, $theme['roles']);
            /**
             * @see https://www.php.net/manual/fr/function.preg-match.php
             */
            preg_match($theme['pattern'], $requestUri, $matches);

            /**
             * Double check about the role of the current user and if the pattern of the URL should changed the current theme
             */
            if (!empty($intersect)
            && !empty($matches)) {
                $this->theme = $theme['theme_code'];
                return true;
            }
        }
        
        /**
         * At least, the current user & URL does not correspond to the changement of the theme, default theme will be used
         */
        return false;
    }

    /**
     * Determine Active Theme
     *
     * @param RouteMatchInterface $route_match
     * 
     * @return string
     */    
    public function determineActiveTheme(RouteMatchInterface $route_match) {
        return $this->theme;
    }
}

==> switch_theme-1.1.0.tar.gz: switch_theme/src/Theme/ThemeNegotiator.php <==
<?php

namespace Drupal\switch_theme\Theme;

use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Theme\ThemeNegotiatorInterface;
use Drupal\switch_theme\SwitchthemeManager;

class ThemeNegotiator implements ThemeNegotiatorInterface {
    /**
     * Theme to negociate
     * 
     * @var string
     */
    protected $theme = "";

    /**
     * Role code of administrator
     * 
     * @todo can be changed, must get it with a query.
     * 
     * @var string
     */
    protected $administratorRoleCode = 'administrator';

    /**
     * The switch theme manager.
     *
     * @var SwitchthemeManager
     */
    protected $switchemeManager;

    /**
     * Construct
     *
     * @param SwitchthemeManager $switchemeManager The switch theme manager
     * 
     * @return void
     */
    public function __construct(SwitchthemeManager $switchemeManager) {
        $this->switchemeManager = $switchemeManager;
    }

    /**
     * Applies theme conditions, "return TRUE" is not a mistake, we want to leave the method as soon as possible.
     *
     * @param RouteMatchInterface $route_match
     * 
     * @return boolean
     */
    public function applies(RouteMatchInterface $route_match) {

        $current_user = \Drupal::currentUser();
        /**
         * @todo could be a memory leak.
         */
        $userRoles = $current_user->getRoles();

        /**
         * The current user is an administrator, the theme will be the default admin theme choosed
         */
        if (in_array($this->administratorRoleCode, $userRoles)) {
            $this->theme = \Drupal::config('system.theme')->get('admin');
            return true;
        }

        /**
         * @todo could be a memory leak.
         */
        $switchthemes = $this->switchemeManager->getSwitchThemes();

        /**
         * Get query parameters to be sure ThemeNegociator will apply or not the change of the theme
         * 
         * @see https://www.php.net/manual/fr/function.strtok.php
         */
        $requestUri = strtok(\Drupal::request()->getRequestUri(), '?');

        /**
         * @todo could be XSS hack opportunity normally not with previous verifications.
         */
        foreach ($switchthemes as $theme) {
            /**
             * @see https://www.php.net/manual/fr/function.array-intersect.php
             */
            $intersect = in_array($theme['role_code'], $userRoles);
            /**
             * @see https://www.php.net/manual/fr/function.preg-match.php
             */
            preg_match($theme['pattern'], $requestUri, $matches);

            /**
             * Double check about the role of the current user and if the pattern of the URL should changed the current theme
             */
            if (!empty($intersect)
            && !empty($matches)) {
                $this->theme = $theme['theme_code'];
                return true;
            }
        }
        
        /**
         * At least, the user does not correspond to the changement of the theme, default theme will be used
         */
        return false;
    }

    /**
     * Determine Active Theme
     *
     * @param RouteMatchInterface $route_match
     * 
     * @return string
     */    
    public function determineActiveTheme(RouteMatchInterface $route_match) {
        return $this->theme;
    }
}
